use strict ;
use warnings ;

# based on standard perl distribution (activeperl 5.6.1 build 635)
#  the following packages are required to be installed separately
#    Image-Magick, Config-ini
use Cwd;
use Image::Magick; 
use File::Basename;
use File::Copy;
use File::Find;
use File::Spec;
use File::Temp qw/ tempfile tempdir /;
use Getopt::Std;
use HTML::Entities;
use HTML::Tree ;
use HTML::TreeBuilder;
use Sys::Hostname ;
use URI ;
use URI::file;
use Text::Wrap ;

#for the graphical version only:
use Config::Ini;
use Tk;
use Tk::Dialog;
use Tk::DialogBox;

#version of this file, calling script or library. Variable not shared. version v, numeric, decimal 2 digits, 00.00 <= v <= 99.99
my $STD_Version=sprintf("%.2f",   1.01       );


sub abs_2_rel
  {
  my $X_pwd=cwd;
  my $ret=undef;

  my $F=$_[0];        # belphegor.alb
  my $D=dirname($F);  # '.'
  my $f=basename($F); # belphegor.alb

  if ( -d $F)
    {
    chdir ($F);
    $ret=cwd;
    }
  elsif ( -d $D )
    {
    chdir ($D);
    $ret = File::Spec->catfile (cwd, $f) ; 
    }
  else
    {
    die ("Error 0249: Path $F does not exist\n");
    }

  chdir ($X_pwd);
  return $ret;

  }




my %Options ;	       # Hash. User options


my $MEID='I'; #Module error ID
my $Caller_Version=$STD_Version ;


  {
  my $Lib_Script='albalib.pl';
  my $Caller_Script=basename(abs_2_rel($0));
  my $Caller_ScriptDir=dirname(abs_2_rel($0));
  my $AlbaLibPath_p = File::Spec->catfile ($Caller_ScriptDir, $Lib_Script) ;
  print STDERR "Unable to run $AlbaLibPath_p: $!\n" if ( ! -r $AlbaLibPath_p) ;
  do "$AlbaLibPath_p" || die "Unable to do $AlbaLibPath_p: $!$@" ; 

  init_guts('guts_product_name_indexer',$Caller_Script, $Caller_ScriptDir, $Lib_Script);
  }


sub bHelp
  {
  print OnlineDoc($Caller_Version) ;
  }

#####################################################################
#####             Initialization of options (getopts)           #####
#####################################################################
our ($opt_h, $opt_v) = (0, 0) ;
getopts('hv') || die ("Error 0329: Error in options.");
if ($opt_h)
  {
  bHelp ;
  exit ;
  }
if ($opt_v)
  {
  print get_guts('guts_lib_version')."/$Caller_Version\n";
  exit ;
  }

my $mw;
my $mw_help ;

my %interface;
$interface{itf_index_file}='';
$interface{itf_home_dir}='';
$interface{itf_masks}='';
$interface{itf_is_new_index}=0;
my $index_root='' ;
my $ini = new Config::Ini(get_guts('guts_inifile'));


init_default_options(\%Options);

load_graphic_ini();

OrganizeGUI();

MainLoop;

# the code below will only be executed after all windows are closed.
save_graphic_ini() ;


sub FileExit
  {
  $mw->destroy  ;
  if ( defined($mw_help) && ref($mw_help) eq 'MainWindow' && Exists($mw_help))
    { 
    $mw_help->destroy() ;
    }
  }
sub BrowseIndex
  { # Button function, no parameters
  $index_root=dirname($interface{itf_index_file}) if ($interface{itf_index_file} ne '') ;
  my $F=$mw->getOpenFile(-initialdir=>dirname($index_root),
    -defaultextension => '.html', 
    -filetypes=>[
                 ['HTML index','.html'],
                 ['All files','*']
                ]);
  if (defined($F))
    {
    $interface{itf_index_file} = $F;
    $index_root=dirname($F);
    }
  }

#sub NewIndex
#  {
#  $interface{itf_index_file}='';
#  # $wd_index_type->configure('state'=>'normal');
#  $interface{itf_is_new_index}=1;
#  }

sub create_index
  {

  my ($inputfile, $outputfile);
  my $itf_index_file=$interface{itf_index_file};
  $itf_index_file =~ s/(\$\{([^}]+)\})/$ENV{$2}/g ;

#########
  #Check the index file (the target file)
  if ( $interface{itf_is_new_index} )
    {
    if ($itf_index_file eq '')
      {
      #new file is NOT specified with interface
      if ( $mw->Dialog(-title => "Index creation",-text =>"Please specify a new index file\nIf you choose an existing file, it will be deleted!!!",  -buttons => ["OK", "Cancel"])->Show  eq 'OK')
        {
DIALOG1:
          {
          my $F = $mw->getSaveFile(
                      -defaultextension => '.html', 
                      -filetypes=>[
                        ['Album','.html'],
                        ['All files','*']
                       ]);
          if (defined($F))
            {
            if ( -f $F )
              {
 
#              $mw->Dialog(-title => "Index creation",-text =>"This index already exist. Please specify a non-existing file for a new index creation",  -buttons => ["Cancel"])->Show;
#              redo DIALOG1;  #restart the block
              # the getSaveFile dialog bow already displayed a warning. We assume the user knows what he is doing...
              unlink ($F) || return $mw->Dialog(-title => "Index creation",-text =>"Create new index failed: unable to delete the index file you selected.",  -buttons => ["Cancel"])->Show;
              }
            $itf_index_file=$interface{itf_index_file}=$F ;
            $index_root=dirname($interface{itf_index_file})
                   #OK: proceed
            }
          else
            {
          #cancel on getSaveFile dialog box
            return ;
            }
          }
        }
      else
        {
        return ;
      #canceled
        }
      }
    else
      {
      #new file is specified with interface
      if ( -f $itf_index_file )
        {
        my $r=$mw->Dialog(-title => "Index creation",-text =>"You selected the creation of a new index on an existing file ($itf_index_file). This file will be erased and replaced by a new standard index",  -buttons => ["OK", "Cancel"])->Show;
        return if ($r eq 'Cancel');
        }
      else
        {
      ;
      #OK proceed
        }
      }
#    print ("Create new index $interface{itf_index_file}\n");
    ($inputfile, $outputfile)=('', $itf_index_file);
    #new index, non-existent file specified, refer to options to create a new index.
    }
  else
    {
    # update existing index
    if ($itf_index_file eq '')
      {
      #existing file is NOT specified with interface
      if ( $mw->Dialog(-title => "Index update",-text =>"Please specify a existing index file",  -buttons => ["OK", "Cancel"])->Show  eq 'OK')
        {
DIALOG2:
          {
          my $F = $mw->getOpenFile(
                      -defaultextension => '.html', 
                      -filetypes=>[
                        ['Album','.html'],
                        ['All files','*']
                       ]);
          if (defined($F))
            {
            if ( ! -f $F )
              {
              $mw->Dialog(-title => "Index update",-text =>"This index does not exist. Please specify a existing file for a index update",  -buttons => ["Cancel"])->Show;
              redo DIALOG2;  #restart the block
              }
            else
              {
              $itf_index_file=$interface{itf_index_file}=$F ;
              $index_root=dirname($interface{itf_index_file});
                   #OK: proceed
              }
            }
          else
            {
          #cancel on getSaveFile dialog box
            return ;
            }
          }
        }
      else
        {
        return ;
      #canceled
        }
      }
    else
      {
      #file is specified with interface
      if ( ! -f $itf_index_file )
        {
        $mw->Dialog(-title => "Index update",-text =>"The index file you specified ($itf_index_file) does not exist. Please specify a existing file for a index update",  -buttons => ["Cancel"])->Show;
        return ;
        }
      else
        {
      ;
      #OK proceed
        }
      }
#    print ("Update index $interface{itf_index_file}\n");
    # update index, existing file specified.
    ($inputfile, $outputfile)=($itf_index_file, $itf_index_file);

    }

#########
  #Check the index file (the target file)

  my @ELOM=expand_list_of_masks($interface{itf_home_dir}, $interface{itf_masks});

#  print ("elom=", @ELOM, "<- (", $#ELOM, "): '$ELOM[0]'") if ($ELOM[0] eq '');
  if ($#ELOM +1 == 0 )
    {
    return undef if ($mw->Dialog(-title => "Index update",-text =>"The list of files is empty. Do you want to proceed anyway?",  -buttons => ["OK", "Cancel"])->Show eq 'Cancel')
    }

  my $r=create_index_of_albums (\@ELOM, $inputfile, $outputfile, $interface{itf_home_dir}) ;
  $mw->Dialog(-title => "Error during index generation",-text =>get_error(0),  -buttons => ["OK"])->Show if (! defined($r));
  return $r;

  }

sub BrowseHomeDir
  {
  ($mw->Dialog(-title => "Select home directory",-text =>"Please select any file in the home directory to select it",  -buttons => ["OK", "Cancel"])->Show eq 'OK')  || return undef;
  my $homedir=$interface{itf_home_dir} ;
  $homedir =~ s/(\$\{([^}]+)\})/$ENV{$2}/g ;
  my $F=$mw->getOpenFile(-initialdir=>$homedir,
    -defaultextension => '.alb', 
    -filetypes=>[
                 ['HTML index','.alb'],
                 ['All files','*']
                ]);
  if (defined($F))
    {
    $interface{itf_home_dir}=dirname($F);
    }
  
  }

sub ExpandMasks
  {

  my @ELOM=(expand_list_of_masks($interface{itf_home_dir}, $interface{itf_masks}));
    # if error, then expand_list_of_masks will return undef and @ELOM will be: (undef) 

    my $homedir=$interface{itf_home_dir};
    $homedir =~ s/(\$\{([^}]+)\})/$ENV{$2}/g ;

  $homedir='.' if ( $homedir eq '');
  $homedir=abs_2_rel($homedir); 

  if ($#ELOM +1 == 0)
    {
    return $mw->Dialog(-title => "List of albums from $homedir",
        -text =>"List is empty (no files match your selection)",  -buttons => ["OK"])->Show;
    }
  elsif (! defined ($ELOM[0])) 
    {
    return $mw->Dialog(-title => "Unable to compute masks",-text =>get_error(0),  -buttons => ["Cancel"])->Show;
    }

  my $l='';
  foreach (@ELOM) { $l=$l."\n".$_; }
  
  $l=substr($l,1) if length($l);

  my $r=$mw->Dialog(-title => "List of albums from $homedir",
        -text =>"List of album to index:\n$l",  -buttons => ["OK", "Open text pad"])->Show;

  open_text_pad("List of albums from $homedir", $l) if ($r eq "Open text pad") ;
  
  }

sub open_text_pad
  {
  my $mw_text;
  my ($title, $text)=(@_);
  $mw_text=MainWindow->new(-title=>$title);
  my $wd_scrolledtext=$mw_text->Scrolled('Text',-scrollbars=>'oe')->pack('expand'=>1, 'fill'=>'both');
  $wd_scrolledtext->configure(-wrap=>'word');
  $wd_scrolledtext->insert('end',$text);
  return $mw_text;
  }

sub load_graphic_ini
  {
#load the parameters from INI with specification of the default values.
  $interface{itf_index_file}= IniGetOrDefault($ini,'index', 'index_file', "");
  $interface{itf_home_dir}= IniGetOrDefault($ini,'index', 'home_dir', "");
  $interface{itf_masks}= IniGetOrDefault($ini,'index', 'masks', "");
  $interface{itf_is_new_index}= 0;
  $index_root=IniGetOrDefault($ini,'index', 'index_root', dirname($interface{itf_index_file}));
  }
sub save_graphic_ini
  {
  $ini->put(['index', 'index_file', $interface{itf_index_file}]);
  $ini->put(['index', 'home_dir', $interface{itf_home_dir}]);
  $ini->put(['index', 'masks', $interface{itf_masks}]);
  $ini->put(['index', 'index_root', $index_root]);

  $ini->put(['scripts', 'line_cmd', get_guts('guts_product_name_line_cmd')]);
  $ini->put(['scripts', 'graphical', get_guts('guts_product_name_graphical')]);
  $ini->put(['scripts', 'indexer', get_guts('guts_product_name_indexer')]);
  $ini->put(['scripts', 'lib', get_guts('guts_product_lib')]);
  $ini->put(['scripts', 'ftp', get_guts('guts_ftp_script')]);

#  $ini->put(['versions', 'line_cmd', '?']);
#  $ini->put(['versions', 'graphical', '?']);
  $ini->put(['versions', 'indexer', $Caller_Version]);
  $ini->put(['versions', 'lib', get_guts('guts_lib_version')]);
#  $ini->put(['versions', 'ftp', '?']);

  $ini->save(get_guts('guts_inifile'));  
  }


sub view_doc
  {
  $mw_help=MainWindow->new(-title=>'Help');
  my $wd_helptext=$mw_help->Scrolled('Text',-scrollbars=>'oe')->pack('expand'=>1, 'fill'=>'both');
  $wd_helptext->configure('wrap'=>'word');
#  my $wd_helptext=$mw_help->Text('wrap'=>'word')->pack('expand'=>1, 'fill'=>'both');
  $wd_helptext->insert('end',OnlineDoc($Caller_Version));
  $wd_helptext->configure('state'=>'disabled');
  }

sub IniGetOrDefault
  {
 # print STDERR (@_);
  
  die if ($#_ +1 ne 4) ;
  
    my $ini=$_[0];
    my $section=$_[1];
    my $key=$_[2];
    my $default=$_[3];

  my $v = $ini->get([$section,$key], -mapping=>'single');
  return (defined $v)?$v:$default;
  }

sub OrganizeGUI
  {
  $mw=MainWindow->new(-title=>get_guts(get_guts('calling_script_ID')));

    #catch the closing of the windows with x button or ALT-F4 or else...
  $mw->wm("protocol","WM_DELETE_WINDOW",\&FileExit);


#+- MainWindow $mw -----------------------------------------------------+
#|+- Frame $B ---------------------------------------------------------+|
#||    Button action frames                                            ||
#|+--------------------------------------------------------------------+|
#|+- Frame $A ---Contain the album parameters--------------------------+|
#||+-- Frame Ap (parameters) ----------------------------------------+ ||
#|||+---$Ap1-------------------------------+                         | ||
#||||cur_dir [ entry ] [button]            |                         | ||
#||||                                      |                         | ||
#||||                                      |                         | ||
#||||                                      |                         | ||
#|||+--------------------------------------+                         | ||
#|||+---$Ap2-------------------------------+                         | ||
#||||                                      |                         | ||
#||||                                      |                         | ||
#||||                                      |                         | ||
#||||                                      |                         | ||
#|||+--------------------------------------+                         | ||
#|||                                                                 | ||
#|||+---$Ap3-------------------------------+                         | ||
#||||                                      |                         | ||
#||||                                      |                         | ||
#||||                                      |                         | ||
#||||                                      |                         | ||
#|||+--------------------------------------+                         | ||
#|||                                                                 | ||
#|||                                                                 | ||
#|||+---$Ap4-------------------------------+                         | ||
#||||+$AP1_Left-++$AP1_Mid---++$AP1_Right+ |                         | ||
#|||||label     ||param1     ||btn 1     | |                         | ||
#|||||label     ||param2     ||btn 2     | |                         | ||
#|||| .                                    |                         | ||
#|||+--------------------------------------+                         | ||
#|||                                                                 | ||
#|||                                                                 | ||
#|||                                                                 | ||
#||+-----------------------------------------------------------------+ ||
#||                                                                    ||
#||+-- Frame Ao (options) -------------------------------------------+ ||
#|||+---$Ao1---------+                                               | ||
#||||o  checkbutton  |                                               | ||
#||||o  checkbutton  |                                               | ||
#||||.               |                                               | ||
#||||.    (options)  |                                               | ||
#||||.               |                                               | ||
#||||                |                                               | ||
#|||+----------------+                                               | ||
#||+-----------------------------------------------------------------+ ||
#||                                                                    ||
#|+--------------------------------------------------------------------+|
#+----------------------------------------------------------------------+

#+- MainWindow $mw -----------------------------------------------------+
  my $B=$mw->Frame(-borderwidth=>2 , -relief=>'sunken')->pack(-side=>'top', -fill=>'x');
  my $A=$mw->Frame(-borderwidth=>2 , -relief=>'sunken')->pack(-side=>'top', -fill=>'x');

#|+- Frame $B ---------------------------------------------------------+|
#||    Button action frames                                            ||
#|+--------------------------------------------------------------------+|

  $B->Button(-text=>'Create/update album', 
    -command=>sub {create_index()})->pack(-side=>'left', -anchor=>'w');

#  $B->Button(-text=>'New', 
#    -command=>sub {NewIndex()})->pack(-side=>'left', -anchor=>'w');

  $B->Button(-text=>'View Doc', 
    -command=>sub {view_doc()})->pack(-side=>'left', -anchor=>'w');


#|+- Frame $A ---Contain the album parameters--------------------------+|
#||+---$A1---++---$A2--+                                               ||
#|||         ||        |                                               ||
#|||         ||        |                                               ||

  my $A0=$A->Frame()->pack(-side=>'top', -fill=>'x');


  my $Ap=$A->Frame(-borderwidth=>2 , -relief=>'sunken')->pack(-side=>'top', -fill=>'x', -expand=>1);
  my $Ap1=$Ap->Frame()->pack(-side=>'top', -anchor=>'nw', -expand=>0);
  my $Ap2=$Ap->Frame()->pack(-side=>'top', -anchor=>'nw', -expand=>0);
  my $Ap3=$Ap->Frame()->pack(-side=>'top', -anchor=>'nw', -expand=>0);
  my $Ap4=$Ap->Frame()->pack(-side=>'top', -anchor=>'nw', -expand=>0);

  my $Ao=$A->Frame(-borderwidth=>2 , -relief=>'sunken')->pack(-side=>'top', -fill=>'x', -expand=>1);

  my $Ao1=$Ao->Frame()->pack(-side=>'left', -expand=>0, -anchor=>'nw');

  $Ao1->Checkbutton(-variable=>\$interface{itf_is_new_index},-text=>"Create new index")->pack(-anchor=>'nw');

  my $Ap4_L=$Ap4->Frame()->pack(-side=>'left');
  my $Ap4_M=$Ap4->Frame()->pack(-side=>'left');
  my $Ap4_R=$Ap4->Frame()->pack(-side=>'left', -fill=>'x', -expand=>1);

  $Ap1->Label(-text=>"Current directory", -width=>15)->pack(-anchor=>'e', -side=>'left'); 
  $Ap1->Entry (-textvariable=>\$interface{itf_home_dir},-width=>75)->pack(-anchor=>'w', -side=>'left');
  $Ap1->Button (-text=>'Browse', -height=>1, -command=>sub {BrowseHomeDir()})->pack(-anchor=>'w', -side=>'left');

  $Ap2->Label(-text=>"Files", -width=>15)->pack(-anchor=>'e', -side=>'left'); 
  $Ap2->Entry (-textvariable=>\$interface{itf_masks}, -width=>75)->pack(-anchor=>'w', -side=>'left');
  $Ap2->Button (-text=>'View list (use wildcards *, ?...)', -height=>1, -command=>sub {ExpandMasks()})->pack(-anchor=>'w', -side=>'left');

  $Ap3->Label(-text=>"Index file", -width=>15)->pack(-anchor=>'e', -side=>'left'); 
  $Ap3->Entry (-textvariable=>\$interface{itf_index_file},-width=>75)->pack(-anchor=>'w', -side=>'left');
  $Ap3->Button (-text=>'Open', -height=>1, -command=>sub {BrowseIndex()})->pack(-anchor=>'w', -side=>'left');
  }

