
use strict ;
use warnings ;

# based on standard perl distribution (activeperl 5.6.1 build 635)
#  the following packages are required to be installed separately
#    Image-Magick, Config-ini
use Cwd;
use Image::Magick; 
use File::Basename;
use File::Copy;
use File::Find;
use File::Spec;
use File::Temp qw/ tempfile tempdir /;
use Getopt::Std;
use HTML::Entities;
use HTML::Tree ;
use HTML::TreeBuilder;
use Sys::Hostname ;
use URI ;
use URI::file;
use Text::Wrap ;

#for the graphical version only:
use Config::Ini;
use Tk;
use Tk::Dialog;
use Tk::DialogBox;

#version of this file, calling script or library. Variable not shared. version v, numeric, decimal 2 digits, 00.00 <= v <= 99.99
my $STD_Version=sprintf("%.2f",   2.1       );

sub abs_2_rel
  {
  my $X_pwd=cwd;
  my $ret=undef;

  my $F=$_[0];        # belphegor.alb
  my $D=dirname($F);  # '.'
  my $f=basename($F); # belphegor.alb

  if ( -d $F)
    {
    chdir ($F);
    $ret=cwd;
    }
  elsif ( -d $D )
    {
    chdir ($D);
    $ret = File::Spec->catfile (cwd, $f) ; 
    }
  else
    {
    die ("Error 0249: Path $F does not exist\n");
    }

  chdir ($X_pwd);
  return $ret;

  }

my %Options ;	       # Hash. User options


my $MEID='G'; #Module error ID
my $Caller_Version=$STD_Version;

  {
  my $Lib_Script='albalib.pl';
  my $Caller_Script=basename(abs_2_rel($0));
  my $Caller_ScriptDir=dirname(abs_2_rel($0));
  my $AlbaLibPath_p = File::Spec->catfile ($Caller_ScriptDir, $Lib_Script) ;
  print STDERR "Unable to run $AlbaLibPath_p: $!\n" if ( ! -r $AlbaLibPath_p) ;
  do "$AlbaLibPath_p" || die "Unable to do $AlbaLibPath_p: $!$@" ; 

  init_guts('guts_product_name_graphical',$Caller_Script, $Caller_ScriptDir, $Lib_Script);

  }



sub bHelp
  {
  print OnlineDoc($Caller_Version) ;
  }

#####################################################################
#####             Initialization of options (getopts)           #####
#####################################################################
our ($opt_h, $opt_v) = (0, 0) ;
getopts('hv') || die ("Error 0329: Error in options.");
if ($opt_h)
  {
  bHelp ;
  exit ;
  }
if ($opt_v)
  {
  print get_guts('guts_lib_version')."/$Caller_Version\n";
  exit ;
  }




my %ConfigTable;       # will contain the album options after we read them from the file
		       # Not expanded
		       # This is shared a global variable
my %Ghost ;  #copy of the config as it is on disk, to ba able to compute the dirty flag

my $mw;
my $mw_help ;
my $wd_pwd ; # widget that contains password
my $itf_Initial_dir='';

my %interface;
$interface{itf_Album_file}='';
$interface{itf_krypt_key}='';
$interface{itf_mandatory_cryption}=1;
$interface{itf_submit_crypted}='';
$interface{itf_crypt_tool_result}='';
$interface{itf_submit_clear}='';

my $ini = new Config::Ini(get_guts('guts_inifile'));


init_default_options(\%Options);

my $guts_ftp_script=get_guts('guts_ftp_script');
my $guts_def_language=get_guts('guts_def_language');


load_graphic_ini();
load_this_album($interface{itf_Album_file}) if ( -f $interface{itf_Album_file});

OrganizeGUI();

MainLoop;

# the code below will only be executed after all windows are closed.
save_graphic_ini() ;

#  my $Ac=$A->Frame(-borderwidth=>2 , -relief=>'sunken')->pack(-side=>'top', -fill=>'x');
#  $Ac->Label (-text=>"Decrypt tool : ", -height=>1.9) ->pack(-side=>'left');
#  $Ac->Entry (-textvariable=>\$interface{itf_submit_crypted})->pack(-side=>'left', -fill=>'both', -expand=>1);
#  $Ac->Button(-text=>'->decrypt->', -height=>1, -command=>sub {DecryptTool('decrypt')})->pack(-side=>'left');
#  $Ac->Entry (-textvariable=>\$interface{itf_crypt_tool_result})->pack(-side=>'left', -fill=>'both', -expand=>1);
#  $Ac->Button(-text=>'<-encrypt<-', -height=>1, -command=>sub {DecryptTool('encrypt')})->pack(-side=>'left');
#  $Ac->Entry (-textvariable=>\$interface{itf_submit_clear})->pack(-side=>'left', -fill=>'both', -expand=>1);
sub DecryptTool
  {
  my $direction=$_[0];
  my $r;
  if ( $direction eq 'encrypt' )
    {
    $r=album_pwd_encrypt($interface{itf_submit_clear}, $interface{itf_krypt_key});
    if (defined $r)
      {
      $interface{itf_submit_crypted}='';
      $interface{itf_crypt_tool_result}=$r;
      } 
    }
  else # decrypt
    {
    $r=album_pwd_decrypt($interface{itf_submit_crypted}, $interface{itf_krypt_key});
    if (defined $r)
      {
      $interface{itf_submit_clear}='';
      $interface{itf_crypt_tool_result}=$r;
      }
    }
  $mw->Dialog(-title => "Error encryption/decryption",-text =>"Error ". get_error(0),  -buttons => ["OK"])->Show if (! defined $r);

  }

sub ConfigIsDirty()
  {
  #is dirty -> return true
  #is clean -> return false
  my %ref_k;
  get_config_default_set(\%ref_k, 
      $Options{opt_default_thumbsize}, 
      $guts_def_language);
  # compare %ConfigTable et %Ghost
  #   both not defined -> clean
  #   %ConfigTable not defined and %Ghost defined -> dirty

  #   %Ghost not defined and %ConfigTable defined 
  #          %ConfigTable is '' -> clean
  #          %ConfigTable is not '' -> dirty

  #   both defined and identical -> clean
  #the flag is dirty if at least one key is dirty
  # the flag is clean if all flags are clean

  my ($dirty, $clean)=(1, 0);

  KEY: foreach my $k (keys(%ref_k))
    {
    if ( ! defined $ConfigTable{$k} )
      {
      if ( ! defined $Ghost{$k} ) 
        { # both undef
        next KEY ;
        }
      else
        {  #   %ConfigTable not defined and %Ghost defined -> dirty
        return $dirty;
        }
      }
    else
      {    # we know now that $ConfigTable{$k} is defined
      if ( ! defined ($Ghost{$k}) )
        {
        if ($ConfigTable{$k} eq '' ) 
          {         next KEY ; }
        else #we know now that Ghost not defined, ConfigTable is defined and not ''
          { return $dirty ; }
        }
      else
        #we know now that Ghost and ConfigTable are both defined
        {
        if ( $ConfigTable{$k} eq $Ghost{$k} ) 
          { next KEY ;  }
        else
          { return $dirty ; }
        }
      }
    }
  return $clean;
  }

sub FileDiscard
  {
  #compute the dirty flag.
  #clean -> discard is accepted
  #clean -> discard is accepted
  
  if (ConfigIsDirty())
    {
    my $r=$mw->Dialog(-title => "Album has changed",-text =>"Do you want to save the current album?",  -buttons => ["Yes", "No", "Cancel"])->Show;
    if ($r eq 'Yes' )  # we backup and then agree for the discard
      {
      return FileSave() ;  # will switch to SaveAs if filename is emty
      }
    elsif ($r eq 'No' ) # we do NOT backup and then agree for the discard
      {
      return 1 ;
      }
    elsif ($r eq 'Cancel' ) # we do NOT backup and then refuse the discard
      {
      return undef ;
      }
    }
  else
    {
    return 1 ;
    }
  }
sub FileExit
  {
  FileDiscard() || return ;
  $mw->destroy  ;
  if ( defined($mw_help) && ref($mw_help) eq 'MainWindow' && Exists($mw_help))
    { 
    $mw_help->destroy() ;
    }
  }
sub FileOpen
  { # Button function, no parameters

  FileDiscard() || return ;
  my $F=$mw->getOpenFile(-initialdir=>$itf_Initial_dir,
    -defaultextension => '.alb', 
    -filetypes=>[
                 ['Album','.alb'],
                 ['All files','*']
                ]);
  if (defined($F))
    {
    $interface{itf_Album_file} = $F;
    load_this_album($interface{itf_Album_file});
    }
  }

sub BrowseMainDir
  {
  my ($F, $selected_ref, $guts_REF_PATH) = (undef, undef, get_guts('guts_REF_PATH') );
  $mw->Dialog(-title => "Main dir selection",-text =>"Select any picture file on the $guts_REF_PATH directory\nto designate the album directory\n(which last leaf name should be as ...../$guts_REF_PATH).",  -buttons => ["OK"])->Show;

  DIALOG:
    {
    $F=$mw->getOpenFile(-initialdir=>$ConfigTable{MainDir},
      -defaultextension => '.jpg', 
      -filetypes=>[
                 ['Jpeg','.jpg .jpeg'],
                 ['Compuserve','.gif'],
                 ['All files','*']
                ]);
    $selected_ref=dirname($F);
    if (defined($F))
      {
      if (  $guts_REF_PATH ne basename($selected_ref)  )
        {
        $mw->Dialog(-title => "Main dir selection",-text =>"Wrong directory.\nthe main directory should be a subdirectory called '$guts_REF_PATH' of the main directory.",  -buttons => ["Cancel"])->Show;
        redo DIALOG;  #restart the block
        }
      else
        {
        $ConfigTable{MainDir}=dirname($selected_ref);
        }
      }
    last DIALOG;  # exit the block
    }
  }


sub FileSave
  { # Button function, no parameters

  if ( $interface{itf_Album_file} ne '' )
    {
    return save_this_album($interface{itf_Album_file});
    }
  else
    {
    return FileSaveAs() ;
    }
  }

sub FileSaveAs
  { # Button function, no parameters
  
  
  my $F = $mw->getSaveFile(-initialdir=>$itf_Initial_dir,
    -defaultextension => '.alb', 
    -filetypes=>[
                 ['Album','.alb'],
                 ['All files','*']
                ]);
  if (defined($F))
    {
    $interface{itf_Album_file} =$F;
    $itf_Initial_dir=dirname($interface{itf_Album_file});
    return save_this_album($interface{itf_Album_file});
    }
  else
    {
    return undef ;
    }
  }

sub NewAlbum
  {
  my $iNewType=$_[0]; # 0: new empty, 1: new default

  FileDiscard() || return ;

  %Ghost=();

  if ($iNewType == 1)
    {
    get_config_default_set(\%ConfigTable, 
      $Options{opt_default_thumbsize}, 
      $guts_def_language);
    }
  else
    {
    foreach my $k (keys(%ConfigTable))
      {
      $ConfigTable{$k}='';
      }
    }
  $interface{itf_Album_file}='';
  $Options{opt_create_thumbnails}=1;
  }

sub load_this_album
  {
  my $file_alb=$_[0];
  $itf_Initial_dir=dirname($file_alb);
  my %forced_to_def;
  my $r=config_file_read_def($file_alb, \%ConfigTable, $Options{opt_default_thumbsize}, $guts_def_language, \%forced_to_def);

  if (! defined ($r)) #error occured
    {
    %Ghost=();
    }
  else
    {
    %Ghost=%ConfigTable;
    }
  if (defined($mw) ) # GUI is available (we are not in the initializing process)
    {
    if (! defined ($r)) #error occured and GUI is available (we are not in the initializing process)
      {
      $mw->Dialog(-title => "Error during Load",-text =>get_error(0),  -buttons => ["OK"])->Show;
      }
    else
      {
      my @forced=keys(%forced_to_def);
      if ($#forced +1 
                    > 0 ) 
        {
	my $str="The following parameters where missing and forced to the default value:\n";
	foreach (sort(@forced))
	  {
	  $str=$str."$_, ";
	  }
	$str =~ s/..$//; #remove the last 2 char which are ', '
        $mw->Dialog(-title=>'Warning $MEID-1406', -text => $str, -buttons => ["OK"])->Show;

        }
      }
    }


  }

sub save_this_album
  {
  my $file_alb=$_[0];
  my $r=config_file_write($file_alb, \%ConfigTable);
  if (! defined ($r)) #error occured
    {
    $mw->Dialog(-title => "Error during write",-text =>get_error(0),  -buttons => ["OK"])->Show;
    return undef ;
    # Ghost not refreshed
    }
  else
    {
      # Ghost refreshed identical as content of file
    %Ghost=%ConfigTable;
    return 1 ;
    }
  }
  

sub sync_server
  {	#itf_mandatory_cryption  #  Flag. Do not accept synchronisation if this option is set and no password is set.

  my $r=album_sync_tab(\%ConfigTable, $interface{itf_krypt_key}, $interface{itf_mandatory_cryption}) ;
  if (! defined ($r)) #error occured
    {
    $mw->Dialog(-title => "Error during album synchronization",-text =>"Error ". get_error(0),  -buttons => ["OK"])->Show;
    }
  return $r;
  }


sub create_album
  {
  my $r=1;
  do #  will loop infinite if option opt_loop, except in case of error.
    {
    $r=album_create_main(\%ConfigTable, $Options{opt_rebuild_thumbnails}, $Options{opt_delete_orphean_thumbn}, $Options{opt_create_thumbnails}, $Options{opt_new_thumbnails}, $Options{opt_mode_wrap}, $Options{opt_mode_Warning}, $Options{opt_loop}, $Caller_Version) ;
    if (! defined ($r)) #error occured
      {
      $mw->Dialog(-title => "Error during album creation",-text =>"Error ". get_error(0),  -buttons => ["OK"])->Show;
      }
    } until (! defined ($r) || ! $Options{opt_loop} )  ;
  return $r;
  }

sub load_graphic_ini
  {
#load the parameters from INI with specification of the default values.
  $itf_Initial_dir= IniGetOrDefault($ini,'localization', 'initial_dir', "");
  $interface{itf_Album_file}= IniGetOrDefault($ini,'localization', 'album_file', "");

  }
sub save_graphic_ini
  {
  $ini->put(['localization', 'initial_dir', $itf_Initial_dir]);
  $ini->put(['localization', 'album_file', $interface{itf_Album_file}]);

  $ini->put(['scripts', 'line_cmd', get_guts('guts_product_name_line_cmd')]);
  $ini->put(['scripts', 'graphical', get_guts('guts_product_name_graphical')]);
  $ini->put(['scripts', 'indexer', get_guts('guts_product_name_indexer')]);
  $ini->put(['scripts', 'lib', get_guts('guts_product_lib')]);
  $ini->put(['scripts', 'ftp', get_guts('guts_ftp_script')]);

#  $ini->put(['versions', 'line_cmd', '?']);
  $ini->put(['versions', 'graphical', $Caller_Version]);
#  $ini->put(['versions', 'indexer', $Caller_Version]);
  $ini->put(['versions', 'lib', get_guts('guts_lib_version')]);
#  $ini->put(['versions', 'ftp', '?']);

  $ini->save(get_guts('guts_inifile'));  
  }

#sub Set_pwd
#  {
#  $wd_pwd->configure('state'=>'normal');
#
#  my $d = $mw->Dialog(-title => "Type in password", -buttons => ["OK", "Cancel", "Set empty"]);
#  my $wd_e=$mw->Label(-text=>'foo');
#  $d->add($wd_e);
#  my  $r = $d->Show;
#
#  if ( $r eq 'OK')
#    { $interface{itf_krypt_key}='foo' ; }
#  elsif ( $r eq 'Set empty')
#    { $interface{itf_krypt_key}='' ; }
#  $wd_pwd->configure('state'=>'disable');
#  }

sub view_doc
  {
  $mw_help=MainWindow->new(-title=>'Help');
  my $wd_helptext=$mw_help->Scrolled('Text',-scrollbars=>'oe')->pack('expand'=>1, 'fill'=>'both');
  $wd_helptext->configure('wrap'=>'word');
#  my $wd_helptext=$mw_help->Text('wrap'=>'word')->pack('expand'=>1, 'fill'=>'both');
  $wd_helptext->insert('end',OnlineDoc($Caller_Version));
  $wd_helptext->configure('state'=>'disabled');
  }

# example: $ini->get(['IPS', 'VOMS_TG_IP'])
sub IniGetOrDefault
  {
 # print STDERR (@_);
  
  die if ($#_ +1 ne 4) ;
  
    my $ini=$_[0];
    my $section=$_[1];
    my $key=$_[2];
    my $default=$_[3];

  my $v = $ini->get([$section,$key], -mapping=>'single');
  return (defined $v)?$v:$default;
  }

sub OrganizeGUI
  {
  $mw=MainWindow->new(-title=>get_guts(get_guts('calling_script_ID')));

    #catch the closing of the windows with x button or ALT-F4 or else...
  $mw->wm("protocol","WM_DELETE_WINDOW",\&FileExit);


#+- MainWindow $mw -----------------------------------------------------+
#|+- Menu $menu_bar ---------------------------------------------------+|
#||  --File--                                                          ||
#||  +vvvvvvv+                                                         ||
#||  |New    |                                                         ||
#||  |Open   |                                                         ||
#||  |Save   |                                                         ||
#||  |Save as|                                                         ||
#||  +-------+                                                         ||
#|+--------------------------------------------------------------------+|
#|+- Frame $B ---------------------------------------------------------+|
#||    Button action frames                                            ||
#|+--------------------------------------------------------------------+|
#|+- Frame $F ---------------------------------------------------------+|
#|| Label | Album file     | label |  passwd | btn set passwd          ||
#|+--------------------------------------------------------------------+|
#|+- Frame $A ---Contain the album parameters--------------------------+|
#||+-Main dir  $A0 --------------------------------------------------+ || 
#||| label |   button | [ main directory  ]                          | ||                                      
#||+-----------------------------------------------------------------+ || 
#||+-Crypt tool $Ac -------------------------------------------------+ || 
#||| label |   button | [ main directory  ]                          | ||                                      
#||+-----------------------------------------------------------------+ || 
#||+-- Frame Ao (options) -------------------------------------------+ ||
#|||+---$Ao1---------++---$Ao2----------------+                      | ||
#||||o  checkbutton  || o  checkbutton        |                      | ||
#||||o  checkbutton  || o  checkbutton        |                      | ||
#||||.               || .                     |                      | ||
#||||.    (options)  || .    (options)        |                      | ||
#||||.               || .                     |                      | ||
#||||                ||                       |                      | ||
#|||+----------------++-----------------------+                      | ||
#||+-----------------------------------------------------------------+ ||
#||+-- Frame Ap (parameters) ----------------------------------------+ ||
#|||+---$Ap1---------++---$Ap2---------++---$Ap3---------+           | ||
#||||+$AP1_L++$AP1_R+||+$AP1_L++$AP1_R+||+$AP1_L++$AP1_R+|           | ||
#|||||label ||param1||||label ||param3||||label ||param5||           | ||
#|||||label ||param2||||label ||param4||||label ||param6||           | ||
#|||| .              || .              || .              |           | ||
#|||| .              || .              || .              |           | ||
#|||| .              || .              || .              |           | ||
#|||+----------------++----------------++----------------+           | ||
#||+-----------------------------------------------------------------+ ||
#||                                                                    ||
#||                                                                    ||
#||                                                                    ||
#||                                                                    ||
#||                                                                    ||
#|| +---$A1---++---$A2--+                                              ||
#|| |         ||        |                                              ||
#|| |         ||        |                                              ||
#||                                                                    ||
#||                                                                    ||
#|+--------------------------------------------------------------------+|
#+----------------------------------------------------------------------+

#On start: album parameters are empty.
#On action "Open file"
#  we discard the current then load the parameters on the interface

#On action "Save file"
#  we write the parameters from the interface in the config file

#On action "Save file as"
#  we write the parameters from the interface in the config file for which the name will be prompted

#On action "New album:
#  we discard the current then we create new album, empty bor with default value

#On action quit:
#  we discard the current then we quit

#Before we discard the value, wither it is on load, new or quit, we test:
#  if album was modified, we ask if backup is required.
#  for that, we keep a dirty flag.
#  the dirty flag will be refresh on demand
#  the dirty flag is computed by comparing the content of the config on disk (from last load/save time) and the content in memory.
#    for that purpose, we keep a copy of the disk in memory. We call that copy: the ghost
#    that copy is initialized on load as the content of the file
#    that copy is initialized on write as the content of the memory
#    that copy is initialized on 'New' as empty



#+- MainWindow $mw -----------------------------------------------------+
  my $menu_bar=$mw->Frame(-relief=>'groove', -borderwidth=>3)->pack(-side=>'top', -fill=>'x');
  my $B=$mw->Frame(-borderwidth=>2 , -relief=>'sunken')->pack(-side=>'top', -fill=>'x');
  my $F=$mw->Frame(-borderwidth=>2 , -relief=>'sunken')->pack(-side=>'top', -fill=>'x');
  my $A=$mw->Frame(-borderwidth=>2 , -relief=>'sunken')->pack(-side=>'top', -fill=>'x');

#|+- Menu $menu_bar ---------------------------------------------------+|
  my $mn_item_file=$menu_bar->Menubutton(-text=>'File')->pack(-side=>'left');
  my $mn_item_actions=$menu_bar->Menubutton(-text=>'Actions')->pack(-side=>'left');
  my $mn_item_help=$menu_bar->Menubutton(-text=>'Help')->pack(-side=>'left');
  
  $mn_item_file->command(-label=>'New empty', -command=>sub{NewAlbum(0)});
  $mn_item_file->command(-label=>'New default', -command=>sub{NewAlbum(1)});
  $mn_item_file->command(-label=>'Open', -command=>sub{FileOpen()});
  $mn_item_file->command(-label=>'Save', -command=>sub{FileSave()});
  $mn_item_file->command(-label=>'Save as', -command=>sub{FileSaveAs()});
  $mn_item_file->separator();
  $mn_item_file->command(-label=>'Exit', -command=>sub {FileExit()});
  
  $mn_item_actions->command(-label=>'Create/update album', -command=>sub{create_album()});
  
  $mn_item_actions->command(-label=>'Synchronise server', -command=>sub{sync_server()});
  
  $mn_item_help->command(-label=>'View doc', -command=>sub{view_doc()});

#|+- Frame $B ---------------------------------------------------------+|
#||    Button action frames                                            ||
#|+--------------------------------------------------------------------+|

  $B->Button(-text=>'New empty', 
    -command=>sub {NewAlbum(0)})->pack(-side=>'left', -anchor=>'w');
  $B->Button(-text=>'New default', 
    -command=>sub {NewAlbum(1)})->pack(-side=>'left', -anchor=>'w');
  $B->Button(-text=>'Open', 
    -command=>sub {FileOpen()})->pack(-side=>'left', -anchor=>'w');

  $B->Button(-text=>'Save', 
    -command=>sub {FileSave()})->pack(-side=>'left', -anchor=>'w');
  $B->Button(-text=>'Save as', 
    -command=>sub {FileSaveAs()})->pack(-side=>'left', -anchor=>'w');

  $B->Button(-text=>'Create/update album', 
    -command=>sub {create_album()})->pack(-side=>'left', -anchor=>'w');

  $B->Button(-text=>'Synchronise server', 
    -command=>sub {sync_server()})->pack(-side=>'left', -anchor=>'w');

  $B->Button(-text=>'View Doc', 
    -command=>sub {view_doc()})->pack(-side=>'left', -anchor=>'w');


#|+- Frame $F ---------------------------------------------------------+|
#||    Album file                                                      ||
#|+--------------------------------------------------------------------+|


  
  my $F1=$F->Frame(-borderwidth=>2 , -relief=>'sunken')->pack(-side=>'left', -expand=>1, -fill=>'both');

  $F1->Label(-text=>"Album file: ")->pack(-side=>'left');
  $F1->Label(-textvariable=>\$interface{itf_Album_file})->pack(-side=>'left', -anchor=>'w');

  my $F2=$F->Frame(-borderwidth=>2 , -relief=>'sunken')->pack(-side=>'left', -expand=>1, -fill=>'both');

  $F2->Label(-text=>"Decrypt key: ")->pack(-side=>'left');
  $wd_pwd=$F2->Entry(-textvariable=>\$interface{itf_krypt_key}, -show=>'*')->pack(-side=>'left');

  # $wd_pwd->configure('state'=>'disabled');


#  $F2->Button(-text=>'Set passwd', -height=>1, -command=>sub {Set_pwd()})->pack(-side=>'left');

#|+- Frame $A ---Contain the album parameters--------------------------+|
#||+---$A1---++---$A2--+                                               ||
#|||         ||        |                                               ||
#|||         ||        |                                               ||

  my $A0=$A->Frame()->pack(-side=>'top', -fill=>'x');
  $A0->Label(-text=>"Main dir", -height=>1.9) ->pack(-side=>'left');
  $A0->Button(-text=>'Browse', -height=>1, -command=>sub {BrowseMainDir(0)})->pack(-side=>'left');
  $A0->Entry (-textvariable=>\$ConfigTable{MainDir})->pack(-side=>'left', -fill=>'both', -expand=>1);
#my $x=10;
#  $A0->Entry(-textvariable=>\$x)->pack(-side=>'left');

  my $Ac=$A->Frame(-borderwidth=>2 , -relief=>'sunken')->pack(-side=>'top', -fill=>'x');
  $Ac->Label (-text=>"Decrypt tool : ", -height=>1.9) ->pack(-side=>'left');
  $Ac->Entry (-textvariable=>\$interface{itf_submit_crypted})->pack(-side=>'left', -fill=>'both', -expand=>1);
  $Ac->Button(-text=>'->decrypt->', -height=>1, -command=>sub {DecryptTool('decrypt')})->pack(-side=>'left');
  $Ac->Entry (-textvariable=>\$interface{itf_crypt_tool_result})->pack(-side=>'left', -fill=>'both', -expand=>1);
  $Ac->Button(-text=>'<-encrypt<-', -height=>1, -command=>sub {DecryptTool('encrypt')})->pack(-side=>'left');
  $Ac->Entry (-textvariable=>\$interface{itf_submit_clear})->pack(-side=>'left', -fill=>'both', -expand=>1);


  my $Ao=$A->Frame(-borderwidth=>2 , -relief=>'sunken')->pack(-side=>'top', -fill=>'x', -expand=>1);

  my $Ao1=$Ao->Frame()->pack(-side=>'left', -expand=>0, -anchor=>'nw');
  my $Ao2=$Ao->Frame()->pack(-side=>'left', -expand=>0, -anchor=>'nw');
  my $Ao3=$Ao->Frame()->pack(-side=>'left', -expand=>0, -anchor=>'nw');

  $Ao1->Checkbutton(-variable=>\$Options{opt_rebuild_thumbnails},-text=>"Force rebuild thumbnails")->pack(-anchor=>'nw');
  $Ao1->Checkbutton(-variable=>\$Options{opt_delete_orphean_thumbn},-text=>"Delete unused thumbnails")->pack(-anchor=>'nw');
  $Ao1->Checkbutton(-variable=>\$Options{opt_create_thumbnails},-text=>"Create missing thumbnails")->pack(-anchor=>'nw');
  $Ao2->Checkbutton(-variable=>\$Options{opt_mode_wrap},-text=>"Wrap log coding (advanced)")->pack(-anchor=>'nw');
  $Ao2->Checkbutton(-variable=>\$Options{opt_mode_Warning},-text=>"Warning on unused pictures in log (advanced)")->pack(-anchor=>'nw');
  $Ao2->Checkbutton(-variable=>\$Options{opt_loop},-text=>"Infinite loop")->pack(-anchor=>'nw');
  $Ao3->Checkbutton(-variable=>\$interface{itf_mandatory_cryption},-text=>"Encrypt mandatory")->pack(-anchor=>'nw');


  my $Ap=$A->Frame(-borderwidth=>2 , -relief=>'sunken')->pack(-side=>'top', -fill=>'x', -expand=>1);
  my $Ap1=$Ap->Frame()->pack(-side=>'left', -anchor=>'nw', -expand=>0);
  my $Ap2=$Ap->Frame()->pack(-side=>'left', -anchor=>'nw', -expand=>0);
  my $Ap3=$Ap->Frame()->pack(-side=>'left', -anchor=>'nw', -expand=>0);

  my $Ap1_L=$Ap1->Frame()->pack(-side=>'left');
  my $Ap1_R=$Ap1->Frame()->pack(-side=>'left', -fill=>'x', -expand=>1);

  my $Ap2_L=$Ap2->Frame()->pack(-side=>'left');
  my $Ap2_R=$Ap2->Frame()->pack(-side=>'left', -fill=>'x', -expand=>1);

  my $Ap3_L=$Ap3->Frame()->pack(-side=>'left');
  my $Ap3_R=$Ap3->Frame()->pack(-side=>'left', -fill=>'x', -expand=>1);
  
  my $A1=$A->Frame()->pack(-side=>'left');
  my $A2=$A->Frame()->pack(-side=>'left', -fill=>'x', -expand=>1);
  
  $Ap1_L->Label(-text=>"Height")->pack(-anchor=>'e'); 
  $Ap1_R->Entry (-textvariable=>\$ConfigTable{Height},-width=>15)->pack(-anchor=>'w');
  
  $Ap1_L->Label(-text=>"NbCol")->pack(-anchor=>'e'); 
  $Ap1_R->Entry (-textvariable=>\$ConfigTable{NbCol},-width=>15)->pack(-anchor=>'w');
  
  
  $Ap1_L->Label(-text=>"Language")->pack(-anchor=>'e'); 
  $Ap1_R->Entry (-textvariable=>\$ConfigTable{Language},-width=>15)->pack(-anchor=>'w');
  
  
  $Ap1_L->Label(-text=>"Default new chapter") ->pack(-anchor=>'e');
  $Ap1_R->Entry (-textvariable=>\$Options{opt_new_thumbnails}, -width=>15)->pack(-anchor=>'w');
  
  
  $Ap1_L->Label(-text=>"META generator")->pack(-anchor=>'e'); 
  $Ap1_R->Entry (-textvariable=>\$ConfigTable{META_generator},-width=>15)->pack(-anchor=>'w');
  
  $Ap1_L->Label(-text=>"Location")->pack(-anchor=>'e'); 
  $Ap1_R->Entry (-textvariable=>\$ConfigTable{Location},-width=>15)->pack(-anchor=>'w');
  
  $Ap2_L->Label(-text=>"Thumbnail surface")->pack(-anchor=>'e');
  $Ap2_R->Entry (-textvariable=>\$ConfigTable{ThumbnailSurface},-width=>15)->pack(-anchor=>'w');

  $Ap2_L->Label(-text=>"FontFace")->pack(-anchor=>'e'); 
  $Ap2_R->Entry (-textvariable=>\$ConfigTable{FontFace},-width=>15)->pack(-anchor=>'w');

  $Ap3_L->Label(-text=>"BODY bgcolor")->pack(-anchor=>'e'); 
  $Ap3_R->Entry (-textvariable=>\$ConfigTable{BODY_bgcolor},-width=>15)->pack(-anchor=>'w');
  
  $Ap3_L->Label(-text=>"BODY text")->pack(-anchor=>'e'); 
  $Ap3_R->Entry (-textvariable=>\$ConfigTable{BODY_text},-width=>15)->pack(-anchor=>'w');
  
  $Ap3_L->Label(-text=>"BODY link")->pack(-anchor=>'e'); 
  $Ap3_R->Entry (-textvariable=>\$ConfigTable{BODY_link},-width=>15)->pack(-anchor=>'w');
  
  $Ap3_L->Label(-text=>"BODY vlink")->pack(-anchor=>'e'); 
  $Ap3_R->Entry (-textvariable=>\$ConfigTable{BODY_vlink},-width=>15)->pack(-anchor=>'w');
  
  $Ap3_L->Label(-text=>"BODY alink")->pack(-anchor=>'e'); 
  $Ap3_R->Entry (-textvariable=>\$ConfigTable{BODY_alink},-width=>15)->pack(-anchor=>'w');
  
  $Ap3_L->Label(-text=>"BODY background")->pack(-anchor=>'e'); 
  $Ap3_R->Entry (-textvariable=>\$ConfigTable{BODY_background},-width=>15)->pack(-anchor=>'w');
  
  $Ap2_L->Label(-text=>"remote host")->pack(-anchor=>'e'); 
  $Ap2_R->Entry (-textvariable=>\$ConfigTable{remote_host},-width=>15)->pack(-anchor=>'w');
  
  $Ap2_L->Label(-text=>"remote user")->pack(-anchor=>'e'); 
  $Ap2_R->Entry (-textvariable=>\$ConfigTable{remote_user},-width=>15)->pack(-anchor=>'w');
  
  $Ap2_L->Label(-text=>"remote root")->pack(-anchor=>'e'); 
  $Ap2_R->Entry (-textvariable=>\$ConfigTable{remote_root},-width=>15)->pack(-anchor=>'w');
  
  $Ap2_L->Label(-text=>"remote pwd")->pack(-anchor=>'e'); 
  $Ap2_R->Entry (-textvariable=>\$ConfigTable{remote_pwd},-width=>15)->pack(-anchor=>'w');

  $A1->Label(-text=>"Main title")->pack(-anchor=>'e');
  $A2->Entry (-textvariable=>\$ConfigTable{TitrePrincipal})->pack(-anchor=>'w', -fill=>'x', -expand=>1);
  
  $A1->Label(-text=>"Tailer"
#, -height=>3
            )->pack(-anchor=>'e');
  $A2->Entry (-textvariable=>\$ConfigTable{Tailer})->pack(-anchor=>'w', -fill=>'x', -expand=>1
#, -ipady=>20
);
  
  $A1->Label(-text=>"META_author")->pack(-anchor=>'e'); 
  $A2->Entry (-textvariable=>\$ConfigTable{META_author          })->pack(-anchor=>'w', -fill=>'x', -expand=>1);
  
  $A1->Label(-text=>"META_description")->pack(-anchor=>'e'); 
  $A2->Entry (-textvariable=>\$ConfigTable{META_description})->pack(-anchor=>'w', -fill=>'x', -expand=>1);
  
  $A1->Label(-text=>"META_distribution")->pack(-anchor=>'e'); 
  $A2->Entry (-textvariable=>\$ConfigTable{META_distribution})->pack(-anchor=>'w', -fill=>'x', -expand=>1);
  
  $A1->Label(-text=>"META_keywords")->pack(-anchor=>'e'); 
  $A2->Entry (-textvariable=>\$ConfigTable{META_keywords})->pack(-anchor=>'w', -fill=>'x', -expand=>1);
  
  $A1->Label(-text=>"META_ownership")->pack(-anchor=>'e'); 
  $A2->Entry (-textvariable=>\$ConfigTable{META_ownership})->pack(-anchor=>'w', -fill=>'x', -expand=>1);

  }
