use strict ;
use warnings ;

# based on standard perl distribution (activeperl 5.6.1 build 635)
#  the following packages are required to be installed separately
#    Image-Magick

use Cwd;
use Image::Magick; 
# use Config::Ini;  I think this one is not required in command-line mode. If so, then update requirements that package Config-ini should be installed
use File::Basename;
use File::Copy;
use File::Find;
use File::Spec;
use File::Temp qw/ tempfile tempdir /;
use Getopt::Std;
use HTML::Entities;
use HTML::Tree ;
use HTML::TreeBuilder;
use Sys::Hostname ;
use URI ;
use URI::file;
use Text::Wrap ;

#version of this file, calling script or library. Variable not shared. version v, numeric, decimal 2 digits, 00.00 <= v <= 99.99
my $STD_Version=sprintf("%.2f",   1.51      );

sub abs_2_rel
  {
  my $X_pwd=cwd;
  my $ret=undef;

  my $F=$_[0];        # belphegor.alb
  my $D=dirname($F);  # '.'
  my $f=basename($F); # belphegor.alb

  if ( -d $F)
    {
    chdir ($F);
    $ret=cwd;
    }
  elsif ( -d $D )
    {
    chdir ($D);
    $ret = File::Spec->catfile (cwd, $f) ; 
    }
  else
    {
    die ("Error 0249: Path $F does not exist\n");
    }

  chdir ($X_pwd);
  return $ret;

  }


my $MEID='N'; #Module error ID
my $Caller_Version=$STD_Version;

#### Common variables between the caller and the library

my %Options ;	       # Hash. User options
my %ConfigTable;       # will contain the album options after we read them from the file


  {
  my $Lib_Script='albalib.pl';
  my $Caller_Script=basename(abs_2_rel($0));
  my $Caller_ScriptDir=dirname(abs_2_rel($0));
  my $AlbaLibPath_p = File::Spec->catfile ($Caller_ScriptDir, $Lib_Script) ;
  print STDERR "Unable to run $AlbaLibPath_p: $!\n" if ( ! -r $AlbaLibPath_p) ;
  do "$AlbaLibPath_p" || die "Unable to do $AlbaLibPath_p: $!$@" ; 

  init_guts('guts_product_name_line_cmd',$Caller_Script, $Caller_ScriptDir, $Lib_Script);

  }


init_default_options(\%Options);

my $guts_ftp_script=get_guts('guts_ftp_script');
my $guts_def_language=get_guts('guts_def_language');

sub bHelp
  {
  print OnlineDoc($Caller_Version) ;
  }

#####################################################################
#####             Initialization of options (getopts)           #####
#####################################################################
our ($opt_f, $opt_h, $opt_w, $opt_I, $opt_S, $opt_t, $opt_F, $opt_v, $opt_W, $opt_i, $opt_H, $opt_T, $opt_s, $opt_R, $opt_P) = 
    ("",     0,      0,      '',   , '',      '',     '',     0,      0,      '',     '',     0,      0,      0    ,   ''   ) ;
getopts('0:f:hist:vwF:H:I:P:RS:TW') || die ("Error in options.");
if ($opt_h)
  {
  bHelp ;
  exit ;
  }
if ($opt_v)
  {
  print get_guts('guts_lib_version')."/$Caller_Version\n";
  exit ;
  }

if ($opt_P ne '')
  {
  print OnlineDoc_package($opt_P);
  exit ;
  }

#check consistency of options
$opt_I || $opt_f || $opt_i || die "Option -f or -I or -i mandatory.\n";

$opt_S && ( $opt_I || die "Option -S requires option -I\n" );

$opt_I && ($opt_f || $opt_i) && die "Option -I, -f and -i exclusive.\n" ;
$opt_f && ($opt_i || $opt_I) && die "Option -f, -i and -I exclusive.\n" ;
$opt_i && ($opt_I || $opt_f) && die "Option -i, -I and -f exclusive.\n" ;

if ( ! $opt_i && $#ARGV+1 > 0 )
  { die "No input arguments allowed with this syntax."; }


#read options values
if ( $opt_f )
  { 
  -r $opt_f || die ("Error T1213: configuration file $opt_f unavailable for read");
  $Options{opt_album_file} = abs_2_rel($opt_f) ; 
  }

$opt_i && ( $Options{opt_create_index_of_album}=1 ) ;
if ($opt_s) {  $Options{opt_mode_silent}=1 ; set_mode_silent();  }  ;

if ( $opt_t ) 
  { 
  $Options{opt_create_thumbnails}=1 ; 
  $Options{opt_new_thumbnails}=$opt_t;
  }

$opt_w && ( $Options{opt_mode_wrap}=1 ) ;

if ( $opt_F )
  { 
  -f $guts_ftp_script || die ("Unable to find script $guts_ftp_script.");
  $Options{opt_with_ftp_sync} = 1 ; 
  $Options{opt_crypt_key} = $opt_F 
  }

if ( $opt_I )
  { 
  -d $opt_I || die ("Unable to find directory $opt_I.");
  $Options{opt_no_ini_start_dir} = rel_2_abs($opt_I) ; 
  my $guts_REF_PATH=get_guts('guts_REF_PATH');
  if ( lc(basename($Options{opt_no_ini_start_dir})) ne lc($guts_REF_PATH) )
    {
    die ("The photo should be in a subdirectory $guts_REF_PATH to initilise album.");
    }
  $Options{opt_no_ini_direct_dir}=1 ; 
  }

$opt_R && ( $Options{opt_delete_orphean_thumbn}=1 ) ;
$opt_S && ( $Options{opt_default_thumbsize}=$opt_S ) ;
$opt_T && ( $Options{opt_rebuild_thumbnails}=1 ) ;
$opt_W && ( $Options{opt_mode_Warning}=1 ) ;



my $ConfigFile; 

#Note: 2 cas: foo.pl -> foo.ini  --Pas foo.pl.ini
#             foo    -> foo.ini   
if ( $Options{opt_no_ini_direct_dir} )
  {
  # we create the config file based on script name
  $ConfigFile=File::Spec->catfile ($Options{opt_no_ini_start_dir}, get_guts('guts_product_name_line_cmd'));
  $ConfigFile =~ s/\.pl$//i ; $ConfigFile = "$ConfigFile.ini" ;
  }
elsif ( $Options{opt_album_file} )
  {
  $ConfigFile=$Options{opt_album_file} ;
  }
elsif ( $Options{opt_create_index_of_album} )
  { 
  if ( ! defined ($opt_H) || $opt_H eq '' )
    { 
    create_index_of_albums(\@ARGV, '', '<STDOUT>', '') || die ("Error during index generation: ".get_error(0));
    }
  else
    {
    create_index_of_albums(\@ARGV, $opt_H, $opt_H, '')  || die ("Error during index creation: ".get_error(0));
    }
  exit ;
  }

print_msg ("Use $ConfigFile\n") ;


#On va lire les paramtres depuis le fichier de config.
#1e: le nom du fichier de config: option -f. Par dfaut: NomDuscript.ini.

if ( $Options{opt_no_ini_direct_dir} && ! -f $ConfigFile)
  {
  #The config file does not already exist
  #Create the first init file based on $AbsStartDir

  get_config_default_set (\%ConfigTable, $Options{opt_default_thumbsize}, $Options{opt_default_language});
    # basename($ConfigTable{MainDir}) = ref
  $ConfigTable{MainDir}=dirname($Options{opt_no_ini_start_dir});
  $ConfigTable{TitrePrincipal}=basename(dirname($ConfigTable{MainDir}));
  $ConfigTable{remote_root}=basename(dirname($ConfigTable{MainDir}));


  #And create the structure (includes: move all files to 'ref'.
#  Initialize_Structure_If_Needed($ConfigTable{MainDir}, $Options{opt_default_thumbsize});


  album_create_main(\%ConfigTable, 
	1, # opt_rebuild_thumbnails
        1, # opt_delete_orphean_thumbn
        1, # opt_create_thumbnails
        'photos', # opt_new_thumbnails
        $Options{opt_mode_wrap}, 
        $Options{opt_mode_Warning},
        $Options{opt_loop},
        $Caller_Version);

  open (INIT_CONFIG_FILE, "> $ConfigFile") || die "Error 0217: unable to initialize $ConfigFile: $!";
  foreach my $k (keys(%ConfigTable))
    {
    print INIT_CONFIG_FILE "$k =$ConfigTable{$k}\n";
    }
  close(INIT_CONFIG_FILE); 

#  init_for_no_ini_direct_dir($Options{opt_no_ini_start_dir}, $ConfigFile, $Options{opt_default_thumbsize})
  }
else
  {
  my ($time1, $time2);

  my %forced_to_def;

  config_file_read_def ($ConfigFile, \%ConfigTable, $Options{opt_default_thumbsize}, $guts_def_language, \%forced_to_def)
    ||
      die ("Error during load: ".get_error(0));
  
  my @forced=keys(%forced_to_def);
  if ($#forced +1 
                  > 0 ) 
    {
    my $str="The following parameters where missing and forced to the default value:\n";
    foreach (sort(@forced))
      {
      $str=$str."$_, ";
      }
    $str =~ s/..$//; #remove the last 2 char which are ', '
    print STDERR "Warning $MEID-1406: $str\n";
    }


  print_msg ("Started: ". get_stampdate()."\n");

  $time1=time();
  
  album_create_main(\%ConfigTable, $Options{opt_rebuild_thumbnails}, 
             $Options{opt_delete_orphean_thumbn}, $Options{opt_create_thumbnails}, 
        $Options{opt_new_thumbnails}, $Options{opt_mode_wrap}, 
        $Options{opt_mode_Warning}, $Options{opt_loop}, $Caller_Version)
    || die ("Error on album creation: ".get_error(0));

  $time2=time();
  print_msg ("Ended: ".get_stampdate()."\nDuration: ".($time2-$time1)."s\n");
  }




if ( $Options{opt_with_ftp_sync} )
  {
  album_sync_file($ConfigFile, $Options{opt_crypt_key}, 0)   #3rd param 0 is option itf_mandatory_cryption used only by GUI
      || die ("Error on sync: ".get_error(0));
#  print "FTP sync\n"; 
#  system ("perl $guts_ftp_script -f $ConfigFile -u -D -x $Options{opt_crypt_key}")
  }

